/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSCLN - some CLN programs to calculate mathematical constants           */
/* =============================================================           */
/*                                                                         */
/* This C++ program is part of the SSCLN package. The program requires an  */
/* installation of the CLN and GMP packages to work. See the references.   */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package from the web-page:                          */
/*                                                                         */
/* http://www.spaennare.se/ssprog.html                                     */
/*                                                                         */
/*                                                                         */
/* References                                                              */
/* ==========                                                              */
/*                                                                         */
/* 1. CLN Class Library for Numbers                                        */
/*                                                                         */
/* http://www.ginac.de/CLN/                                                */
/*                                                                         */
/* 2. GMP A free library for arbitrary precision arithmetic                */
/*                                                                         */
/* http://gmplib.org/                                                      */
/*                                                                         */
/* 3. Mathematical constants and computation                               */
/*                                                                         */
/* http://numbers.computation.free.fr/Constants/constants.html             */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*                                                           */
/*   Author:         Stefan Spaennare                        */
/*   First version:  H2 2003                                 */
/*   Latest updates: 2007-09-26                              */
/*                                                           */
/*************************************************************/


#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include <cln/cln.h>

using namespace std;
using namespace cln;

double timetic=(double)(CLOCKS_PER_SEC);

int main(int argc,char *argv[]) 
{
    
   int i,n,pot;


   if (argc != 2) {
      printf("Usage: %s power-of-2i\n",argv[0]);
      exit(0);
   } /* if */

   pot=atoi(argv[1]);

   n=1;

   for (i=1; i<=pot; i++) {
      n=2*n;
   } /* for i */

   float_format_t prec = float_format(n);

   cl_F result = catalanconst(prec);

   printf("\n");

   print_float(cout,default_print_flags,result);
  
   printf("\n");
   printf("\n");

} /* End */
