/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSCLN - some CLN programs to calculate mathematical constants           */
/* =============================================================           */
/*                                                                         */
/* This C++ program is part of the SSCLN package. The program requires an  */
/* installation of the CLN and GMP packages to work. See the references.   */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package from the web-page:                          */
/*                                                                         */
/* http://www.spaennare.se/ssprog.html                                     */
/*                                                                         */
/*                                                                         */
/* References                                                              */
/* ==========                                                              */
/*                                                                         */
/* 1. CLN Class Library for Numbers                                        */
/*                                                                         */
/* http://www.ginac.de/CLN/                                                */
/*                                                                         */
/* 2. GMP A free library for arbitrary precision arithmetic                */
/*                                                                         */
/* http://gmplib.org/                                                      */
/*                                                                         */
/* 3. Mathematical constants and computation                               */
/*                                                                         */
/* http://numbers.computation.free.fr/Constants/constants.html             */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*                                                           */
/*   Author:         Stefan Spaennare                        */
/*   First version:  H2 2003                                 */
/*   Latest updates: 2007-09-26                              */
/*                                                           */
/*************************************************************/


#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include <cln/cln.h>

using namespace std;
using namespace cln;

double timetic=(double)(CLOCKS_PER_SEC);

int main(int argc,char *argv[]) 
{
    
   int i,j,k,n,pot,s,N,x;


   if (argc != 3) {
      printf("Usage: %s si power-of-2i\n",argv[0]);
      exit(0);
   } /* if */

   s=atoi(argv[1]);
   pot=atoi(argv[2]);

   n=1;

   for (i=1; i<=pot; i++) {
      n=2*n;
   } /* for i */


   x=(int)(0.693148*(double)(4*n))+1;

   N=(int)(exp(1.0)*(double)(x));

   float_format_t prec = float_format(n);


   cl_F temp=cl_float(0,prec);

   cl_F fterm=cl_float(0,prec);

   cl_F fsum=cl_float(0,prec);

   cl_F gterm=cl_float(0,prec);

   cl_F gsum=cl_float(0,prec);


   fterm=cl_float(1,prec);
   
   fsum=fterm;

   gterm=fterm;

   gsum=gterm;


   for (i=1; i<N; i++) {

      temp=cl_float(x,prec)/cl_float(i,prec);

      fterm=fterm*temp;

      fsum=fsum+fterm;

      gterm=gterm*temp;

      if ((i % 2) == 0) {
         k=1;
      }
      else {
         k=-1;
      } /* if */

      temp=cl_float(k*(i+1),prec);

      for (j=1; j<=(s-1); j++) {

         temp=temp*cl_float(i+1,prec);

      } /* for j */

      gterm=gterm+fterm/temp;

      gsum=gsum+gterm;

   } /* for i */

   gsum=gsum/(fsum+cl_float(1,prec));

   temp=cl_float(1,prec);

   for (j=1; j<=(s-1); j++) {

      temp=temp*cl_float(2,prec);

   } /* for j */

   gsum=gsum*temp;

   temp=temp-cl_float(1,prec);

   gsum=gsum/temp;


   printf("\n");

   print_float(cout,default_print_flags,gsum);
  
   printf("\n");
   printf("\n");

} /* End */
